<#
.SYNOPSIS
Deploys a solution package.

.PARAMETER solution
path to solution package.

.PARAMETER webApplication
Web applications to install solutions on,
if solution has Web application resources

.PARAMETER gacDeployment
Deploy to GAC

.PARAMETER casPolicy
Deploy CAS policies

.PARAMETER update
Update a solution instead of
retracting and removing.
#>

param(
[string]$solution,
[array]$webApplication,
[switch]$gacDeployment,
[switch]$casPolicy,
[switch]$update
)

# stops the SPAdminV4 service and runs Start-SPAdminJob
function Restart-SPAdminV4([switch]$adminJob) {
  Stop-Service SPAdminV4
  if($adminJob) { Start-SPAdminJob }
  Start-Service SPAdminV4
}

# Check if Snap-in is loaded
if(-not(Get-PSSnapin | Where { $_.Name -eq "Microsoft.SharePoint.PowerShell"})) {
  Add-PSSnapin Microsoft.SharePoint.PowerShell;
}

# Check if solution exists.
$spSolution = Get-SPSolution -Identity (Split-Path $solution -Leaf) -ErrorActionSilentlyContinue

# Check if solution exists and should be removed

if($spSolution -AND -not($update)) {
  # Check if solution is deployed
  if($spSolution.Deployed) {
    # Check if solution has Web application resources
    Write-Host "Retracting Solution."
    if($spSolution.ContainsWebApplicationResource) {
      $spSolution | Uninstall-SPSolution -AllWebApplications -Confirm:$false
    } else {
      $spSolution | Uninstall-SPSolution -Confirm:$false
    }
  }

  Restart-SPAdminV4 -adminJob

  # Wait for solution
  do { Start-Sleep -Seconds 1 } while ($spSolution.Deployed)

  # Remove solution
  Write-Host "Removing Solution."

  $spSolution | Remove-SPSolution -Confirm:$false

  # nullify $spSolution
  $spSolution = $null
}

# Check if solution does not exist
if(-not($spSolution)) {
  # Add solution
  Write-Host "Adding Solution."
  $spSolution = Add-SPSolution -LiteralPath $solution
}

# check if solution contains Web application resources
if($spSolution.ContainsWebApplicationResource -AND -not($webApplication)) {
  Write-Host "The solution $solution contains Web application resources"
  Write-Host "please specify Web applications."
  return;
}

if(-not($spSolution.Deployed)) {
  # Check if there are application-level resources
  if ($spSolution.ContainsWebApplicationResource) {
    # Install solution on each specified Web application
    foreach($wa in $webapplication) {
      Write-Host "Deploying solution to $wa."
      Install-SPSolution -identity $spSolution.Name -Webapplication $wa -GACDeployment:$gacDeployment -CASPolicies:$casPolicy -force
    }
  } else {
    # Install solution on Farm
    Write-Host "Deploying solution to Farm."
    Install-SPSolution -identity $spSolution.Name -GACDeployment:$gacDeployment -CASPolicies:$casPolicy -force
  }
} else {
  # Check if there are application-level resources
  if ($spSolution.ContainsWebApplicationResource) {
    # Update solution on each specified Web application
    foreach($wa in $webapplication) {
      Write-Host "Updating solution on $wa,"
      Update-SPSolution -identity $spSolution.Name -Webapplication $wa -LiteralPath $solution -GACDeployment:$gacDeployment -CASPolicies:$casPolicy -force
    }
  } else {
    # Update solution on Farm level
    Write-Host "Updating solution on Farm."
    Update-SPSolution -identity $spSolution.Name -LiteralPath $solution -GACDeployment:$gacDeployment -CASPolicies:$casPolicy -force
  }
}

Restart-SPAdminV4 -adminJob

# Wait for solution
do { Start-Sleep -Seconds 1 } while (-not($spSolution.Deployed))