<#
.SYNOPSIS
Automates creation of site collections
based on information stored in a SharePoint 2010 list.

.DESCRIPTION
The script automates creation of site collections
based on a SharePoint List containing various settings
required.

.PARAMETER listURL
URL of the source list.

.PARAMETER field
Name of the field to be used for filtering items, default is set to 'Status'.

.PARAMETER pattern
Value to be used for filtering items, default is set to 'Requested'.
#>

param($listURL,$field = "Status",$pattern = "Requested")

# Check if Snap-in is loaded
If (-not(Get-PSSnapin | Where-Object { $_.Name -eq "Microsoft.SharePoint.PowerShell"})) {
  Add-PSSnapin Microsoft.SharePoint.PowerShell;
}

# Convert special characters to quoted UTF-8 format
$webUrl = [Microsoft.SharePoint.Utilities.SPEncode]::UrlDecodeAsUrl($listUrl);
# Iterate through each segment
-1..-(([uri]$listURL).Segments).Count | ForEach-Object {
  if ($spWeb -eq $null) {
    $webUrl = $webUrl -replace [Microsoft.SharePoint.Utilities.SPEncode]::UrlDecodeAsUrl(([uri]$listURL).Segments[$_]);
    $spWeb = Get-SPWeb -Identity $webUrl -ErrorAction SilentlyContinue
  }
}

# Get the list
$spList = $spWeb.GetList($listURL);

# Store list items in an array
$items = $spList.Items | Where-Object { $_[$field] -eq $pattern }

# Check if the array contains information
if ($items -ne $null) {
  # Loop through list item collection
  $items | ForEach-Object {
    # Build site URL and replace non-alphanumeric characters
    $siteName = $_["Site name"] -Replace "\W","";
    $siteURL = $webURL + "sites/" + $siteName;

    # Check if site collection already exists
    if(Get-SPSite -Identity $siteURL -ErrorAction SilentlyContinue) {
      # Site already exists
      Write-Host "Site $siteURL already exists";

      # Updating comments
      $_["Comment"] = "Site already exists, choose a new name";
      $_.Update();
    } else {
      # Get the user ID
      $userID = $_["Site owner"] -Replace "\d*;#","";

      # Check if User exists
      if(Get-SPUser -Web $webUrl -Identity $userID -ErrorAction SilentlyContinue) {
        $user = Get-SPUser -Web $webUrl -Identity $userID
      } else {
        $user = New-SPUSer -Web $webUrl -UserAlias $userID
      }
      # Create site collection
      New-SPSite -url $siteURL -Name $_["Site name"] -Description $_["Site description"] -OwnerAlias $user -Template "STS#0";

      # Update the list item
      $_["Status"] = "Created";
      $_["Comment"] = "Site $siteURL created";
      $_.Update();
    }
  }
}

# Dispose SPWeb
$spWeb.Dispose()