<#
.SYNOPSIS
Checks if sites have been modified within a given timespan.

.DESCRIPTION
The script checks if sites within a site collection have been modified within a specified timespan. If the switch -sendMail is used, the script sends an email to the address specified in the RequestAccessEmail property.

.PARAMETER url
Site Collection URL.

.PARAMETER days
Days to check, default set to 180.

.PARAMETER sendMail
Sends an email to the address specified in the RequestAccessEmail property.
#>

param(
[string]$url,
[int32]$days = 180,
[switch]$sendMail
)

# Check if Snap-in is loaded
if (-not(Get-PSSnapin | Where-Object { $_.Name -eq "Microsoft.SharePoint.PowerShell"})) {
  Add-PSSnapin Microsoft.SharePoint.PowerShell;
}

# create DateTime object 30 days back.
$lastModified = (Get-Date).Subtract((New-TimeSpan -Days $days))

Get-SPSite $url | Get-SPWeb | ForEach-Object {
  # Check when the last item was modified
  if($_.LastItemModifiedDate -le $lastModified) {
    # Store web information in object
    $objSelected = $_ | Select-Object Url, Title, Description,
      @{Name="Modified";Expression={$_.LastItemModifiedDate}},
      @{Name="email";Expression={$_.RequestAccessEmail}},
      @{Name="inactive";Expression={(Get-date).Subtract($($_.LastItemModifiedDate)).Days}}
    
    # if string is null or empty, use author email instead
    if ([string]::IsNullOrEmpty($objSelected.email)) {
      $objSelected.email = $_.Author.Email
    }
    
    # Check if mail should be sent
    if ($sendMail and $objSelected.email) {
      # Get SMTP Settings.
      $smtp = ((Get-SPWebApplication $url).OutboundMailServiceInstance).Server.Address
      $from = (Get-SPWebApplication $url).OutboundMailSenderAddress
      # Store subject in a variable
      $subject = "The site $($_.Title) has been inactive in $days days"
      
      # Body as html
      $body = [string]::Join("`n", ($objSelected | ConvertTo-Html -Property Url, Title,Description, Modified, email, inactive))
      Send-MailMessage -to $objSelected.email -from $from -Subject $subject -Body $body -SmtpServer $smtp -BodyAsHtml
    }
    # Return information to PowerShell
    return $objSelected
  }
}