<#
.SYNOPSIS
Checks in files in a SharePoint document library.

.DESCRIPTION
The script checks in a single file or loops through a document library and checks in all files.

.PARAMETER url
Site URL.

.PARAMETER folder
Folder/Document library name

.PARAMETER file
File name

.PARAMETER comment
Version comment

.PARAMETER checkinType
Checkin type, can be: Major, Minor or Overwrite

.PARAMETER all
Checks out all files.
#>

param(
[string]$url,
[string]$folder,
[string]$file,
[string]$comment,
[string]$checkinType,
[switch]$all
)

# Check if Snap-in is loaded
if(-not(Get-PSSnapin | Where { $_.Name -eq "Microsoft.SharePoint.PowerShell"})) {
  Add-PSSnapin Microsoft.SharePoint.PowerShell;
}

# Use a switch to get checkin type, defaults to minor version if value not within range.
switch($checkinType) {
  {$_ -match "^major" } { $spCheckinType = [Microsoft.SharePoint.SPCheckinType]::MajorCheckIn; }
  {$_ -match "^minor" } { $spCheckinType = [Microsoft.SharePoint.SPCheckinType]::MinorCheckIn; }
  {$_ -match "^overwrite" } { $spCheckinType = [Microsoft.SharePoint.SPCheckinType]::OverwriteCheckIn; }
  Default { $spCheckinType = [Microsoft.SharePoint.SPCheckinType]::MinorCheckIn; }
}

$spWeb = Get-SPWeb $url

# Check if All files should be checked out
if ($all) {
  # Get the folder
  $spFolder = $spWeb.GetFolder($folder);
  # Store file collection in a variable
  $spFileCollection = $spFolder.Files;
  # Loop through files and check out if file is not already checked out
  $spFileCollection | ForEach-Object {
    # Check if file is not checked in
    if ($_.CheckOutType -ne "None") {
      Try {
        $_.CheckIn($comment,$spCheckinType);
      } Catch {
        "You cannot overwrite a major version file";
        $check = $true;
      } Finally {
        if ($check -eq $true) {
          Write-Host $_.Name Not Checked in;
        } else {
          Write-Host $_.Name checked in;
        }
        $check = $null;
      }
    } else {
      Write-Host $_.Name already checked in;
    }
  }
} else {
  # Store file URL in a variable
  $fileURL = $folder + "/" + $file;
  $spFile = $spWeb.GetFile($fileURL);
  # Check if file is not checked in
  if ($spFile.CheckOutType -ne "None") {
    Try {
      $spFile.CheckIn($comment,$spCheckinType);
    } Catch {
      "You cannot overwrite a major version file";
      $check = $true;
    } Finally {
      if ($check -eq $true) {
        Write-Host $spFile.Name Not Checked in;
      } else {
        Write-Host $spFile.Name checked in;
      }
      $check = $null;
    }
  } else {
    Write-Host $spFile.Name already checked in;
  }
}
$spWeb.Dispose()